use std::{collections::HashMap};

use nanobyte_tera::{l18n::translate_filter, date::{calculate_age, get_year}, string::insert_space_every, gravatar::gravatar_link};
use ovlach_tera::entity::lang_entity;
use rocket::{request::{FromRequest, Outcome}, Request};
use tera::{Tera, Value, Error};

#[derive(Debug)]
pub struct NanoTera(pub Tera);


// Allows a route to access the span
#[rocket::async_trait]
impl<'r> FromRequest<'r> for NanoTera {
    type Error = ();

    async fn from_request(request: &'r Request<'_>) -> Outcome<Self, ()> {
        let mut tera = Tera::new("templates/*").unwrap();
        tera.register_filter("translate", translate_filter);
        tera.register_filter("calculate_age", calculate_age);
        tera.register_filter("insert_space_every", insert_space_every);
        tera.register_filter("gravatar_link", gravatar_link);
        // filters specific to API
        tera.register_filter("lang_entity", lang_entity);
        tera.register_filter("format_date", get_year); // deprecated
        tera.register_filter("get_year", get_year);
        tera.register_filter("strip_proto", strip_proto);
        rocket::outcome::Outcome::Success(NanoTera(tera))
    }
}


/// Strip protocol from URL (value)
pub fn strip_proto(
    tera_value: &Value,
    _: &HashMap<String, Value>
) -> Result<Value, Error> {
    let value = tera_value.as_str().unwrap();
    if value.starts_with("http://") {
        Ok(Value::String(value.strip_prefix("http://").unwrap().to_string()))
    } else if value.starts_with("https://") {
        Ok(Value::String(value.strip_prefix("https://").unwrap().to_string()))
    } else {
        Ok(Value::String(value.to_string()))
    }
}

